<?php 
/**
 * @file
 * Contains \Drupal\commerce_gc_client\Controller\WebhookHandler.
 */

namespace Drupal\commerce_gc_client\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Response;
use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_gc_client\Controller\GoCardlessPartner;
use Drupal\commerce_price\Price;
use Drupal\commerce_payment\Entity\Payment;
use Drupal\commerce_gc_client\Event\GoCardlessEvents;
use Drupal\commerce_gc_client\Event\WebhookEvent;

/**
 * Default controller for the commerce_gc_client module.
 */
class WebhookHandler extends ControllerBase {

  private $db;
  private $log_storage;
  private $payment_storage;

  /**
   * Get Commerce order ID.
   *
   * @param string $id
   *   Either a GoCardless mandate ID or a GoCardless subscription ID.
   *
   * @return int
   *   The Commerce order ID for the mandate or subscription. 
   */
  private function getOrderId($id) {

    //$db = \Drupal::database();
    $query = $this->db->select('commerce_gc_client', 'g');
    if (substr($id, 0, 2) == 'MD') {
      $query->condition('gc_mandate_id', $id);
    }
    else {
      $query->join('commerce_gc_client_item', 'i', 'g.gcid = i.gcid');
      $query->condition('gc_subscription_id', $id);
    }
    return $query->fields('g', array('order_id'))->execute()->fetchField();
  }

  /**
   * Log webhook events.
   *
   * Adds a webhook event to the Drupal log if the feature is enabled in the
   * modules settings form.
   */
  private function log($payment_gateway_settings, $event) {
    if ($payment_gateway_settings['log_webhook']) {
      \Drupal::logger('commerce_gc_client')->notice('<pre>GoCardless webhook: <br />' . print_r($event, TRUE) . '</pre>');
    }
  }

  /**
   * Get payment gateway settings.
   */
  private function getPaymentGatewaySettings($order) {
    $payment_gateway_id = $order->payment_gateway->entity->Id();
    $payment_gateway_settings = GoCardlessPartner::getSettings($payment_gateway_id)['configuration'];
    $payment_gateway_settings['id'] = $payment_gateway_id;
    return $payment_gateway_settings;
  }


  /**
   * Capture and pre-process incoming webhooks from GoCardless.
   */
  public function webhook() {

    $this->db = \Drupal::database();
    $this->log_storage = \Drupal::entityTypeManager()->getStorage('commerce_log');

    //Required for nginx servers.
    if (!function_exists('getallheaders')) {
      function getallheaders() {
        $headers = [];
        foreach ($_SERVER as $name => $value) {
          if (substr($name, 0, 5) == 'HTTP_') {
            $headers[str_replace(' ', '-', ucwords(strtolower(str_replace('_', ' ', substr($name, 5)))))] = $value;
          }
        }
        return $headers;
      }
    }

    $headers = getallheaders();
    $provided_signature = $headers['Webhook-Signature'];
    $calculated_signature = FALSE;
    $module_settings = \Drupal::config('commerce_gc_client.settings')->get();
    strpos($headers['Origin'], 'sandbox') !== false ? $mode = 'sandbox' : $mode = 'live';

    if ($secret = $module_settings['webhook_secret_' . $mode]) {
      $webhook = file_get_contents('php://input');
      $calculated_signature = hash_hmac("sha256", $webhook, $secret);
    }

    if ($provided_signature == $calculated_signature) {

      $data = json_decode($webhook, TRUE);
      $event_dispatcher = \Drupal::service('event_dispatcher');

      // Process the events.
      foreach ($data['events'] as $event) {

        switch ($event['resource_type']) {

          case 'mandates':
            if ($order_id = $this->getOrderId($event['links']['mandate'])) {
              if ($order = Order::load($order_id)) {
                $this->mandates($order, $event);
              }
            }
            break;

          case 'subscriptions':
            if ($order_id = $this->getOrderId($event['links']['subscription'])) {
              if ($order = Order::load($order_id)) {
                $this->subscriptions($order, $event);
              }
            }
            break;

          case 'payments':
            $payment_id = $event['links']['payment'];
            $partner = new GoCardlessPartner();
            $result = $partner->api([
              'mode' => $mode,
              'endpoint' => 'payments',
              'action' => 'get',
              'id' => $payment_id,
            ]);
            if ($result->response->status_code == 200) {
              $resource = $result->response->body->payments;
              if ($order_id = $this->getOrderId($resource->links->mandate)) {
                if ($order = Order::load($order_id)) {
                  $this->payments($order, $event, $resource);
                }
              }  
            }
            break;
        }

        if (isset($order)) {
          $this->log($this->getPaymentGatewaySettings($order), $event);
        }

        //Dispatch an event so that other modules can respond to webhook 
        !isset($resource) ? $resource = NULL : NULL;
        $webhook_event = new WebhookEvent($event, $resource, $order_id);
        $event_dispatcher->dispatch(GoCardlessEvents::WEBHOOK, $webhook_event); 
      }
    }
    else {
      \Drupal::logger('commerce_gc_client')->warning(t('Webhook cannot be processed because the webhook secret is not set or is invalid.'));
      return new Response('Forbidden.', 403, ['Content-Type' => 'text/html']);
    }
    return new Response();
  }

  /**
   * Processes 'mandates' webhooks.
   */
  private function mandates($order, $event) {

    $this->log_storage->generate($order, 'webhook_description', [
      'description' => $event['details']['description'],
    ])->save();

    $this->db->update('commerce_gc_client')->fields([
        'gc_mandate_status' => $event['action'],
      ])->condition('order_id', $order->id())->execute();
  }

  /**
   * Processes 'subscription' webhooks.
   */
  private function subscriptions($order, $event) {

    $this->log_storage->generate($order, 'webhook_description', [
      'description' => $event['details']['description'],
    ])->save();

    if (!in_array($event['action'], ['payment_created', 'amended'])) {
      db_update('commerce_gc_client_item')->fields([
          'gc_subscription_status' => $event['action'],
        ])->condition('gc_subscription_id', $event['links']['subscription'])->execute();
    }
  }

  /**
   * Processes 'payments' webhooks.
   */
  private function payments($order, $event, $payment) {

    if (!$this->payment_storage) {
      $this->payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    }

    if ($event['action'] == 'created') {
      $commerce_payment = $this->payment_storage->create([
        'state' => 'new',
        'amount' => new Price((string)($payment->amount / 100), $payment->currency),
        'payment_gateway' => $order->payment_gateway->entity->Id(),
        'order_id' => $order->id(),
        'remote_id' => $payment->id,
        'remote_state' => $payment->status,
      ])->save();
    }
    else {

      $payment_id = $this->db->select('commerce_payment', 'p')->fields('p', ['payment_id'])->condition('remote_id', $payment->id)->execute()->fetchField();
      $payment = Payment::load($payment_id);

      switch (TRUE) {
        case in_array($event['action'], [
            'pending_customer_approval',
            'pending_submission',
            'submitted',
          ]):
          $payment->setState('authorization');
          break;

        case in_array($event['action'], [
            'confirmed',
            'paid_out',
          ]):
          $payment->setState('completed');
          break;

        case in_array($event['action'], [
            'cancelled',
            'customer_approval_denied',
            'failed',
            'charged_back',
          ]):
          $payment->setState('authorization_voided');
          break;
      }
      $payment->save();
    }
  }
}
