<?php

namespace Drupal\commerce_gc_client\Event;

use Symfony\Component\EventDispatcher\Event;

/**
 * Event that is fired from the Offsite Payment form.
 *
 * Calculates a proportion of the shipment amount to the order item.
 */
class ShipmentEvent extends Event {

  /**
   * The Commerce Shipping shipment entity.
   *
   * @var object
   */
  protected $shipment;

  /**
   * The Commerce order item entity.
   *
   * @var object
   */
  protected $order_item;

  /**
   * If the order item is a shipment item.
   *
   * @var bool
   */
  protected $order_item_shipped = FALSE;

  /**
   * The calculated shipment price for an order item.
   *
   * @var array
   */
  protected $item_shipment_proportion = 0;

  protected $shipment_items_qty = 0;

  /**
   * Constructs the object.
   *
   * @param object $shipment
   *   The Commerce Shipping shipment entity.
   * @param object $order_item
   *   The Commerce order item entity.
   */
  public function __construct($shipment, $order_item) {

    $this->shipment = $shipment;
    $this->order_item = $order_item;

    $order_item_id = $order_item->id();
    foreach($shipment->getItems() as $shipment_item) {
      if ($order_item_id == $shipment_item->getOrderItemId()) {
        $this->order_item_shipped = TRUE;
      }
      $this->shipment_items_qty += $shipment_item->getQuantity();  
    }
  }

  /**
   * Gets the shipment entity.
   *
   * @return object
   *   The Commerce Shipping shipment entity.
   */
  public function getShipment() {
    return $this->shipment;
  }

  public function getShipmentItemsQty() {
    return $this->shipment_items_qty;
  }

  /**
   * Gets the order item shipped boolean.
   *
   * @return bool
   *   If the order item is a shipment item.
   */
  public function getOrderItemShipped() {
    return $this->order_item_shipped;
  }

  /**
   * Gets the order item entity.
   *
   * @return object
   *   The Commerce order item.
   */
  public function getOrderItem() {
    return $this->order_item;
  }

  /**
   * Gets the item shipment price.
   *
   * @return array
   *   The calculated shippment price array.
   */
  public function getItemShipmentProportion() {
    return $this->item_shipment_proportion;
  }

  /**
   * Sets the item shipment price.
   *
   * @param array $item_shipment_price
   *   The calculated shipment price array.
   */
  public function setItemShipmentProportion($item_shipment_proportion) {
    $this->item_shipment_proportion = $item_shipment_proportion;
  }
}
